@@ -1,5 +1,7 @@
 #![cfg(test)]
 
+use std::collections::HashMap;
+
 use super::content_index::ContentIndexEntries;
 use super::*;
 use bytes::{Bytes, BytesMut};
@@ -21,11 +23,12 @@ impl<'a> MockContentEngine<'a> {
     }
 }
 impl<'a> ContentEngine<()> for MockContentEngine<'a> {
-    fn render_context<QueryParameters: Clone + Serialize>(
+    fn render_context(
         &self,
         route: Option<Route>,
-        query_parameters: QueryParameters,
-    ) -> RenderContext<(), QueryParameters, Self> {
+        query_parameters: HashMap<String, String>,
+        request_headers: HashMap<String, String>,
+    ) -> RenderContext<(), Self> {
         RenderContext {
             content_engine: self,
             data: RenderData {
@@ -36,6 +39,7 @@ impl<'a> ContentEngine<()> for MockContentEngine<'a> {
                 request: RequestData {
                     route,
                     query_parameters,
+                    request_headers,
                 },
             },
         }
